@@ -26,12 +26,6 @@ _LOGGER = logging.getLogger('BuildUp.GitBuilding')
 # to be methods. As such disabling no-self-use
 # pylint: disable=no-self-use
 
-def _send_file_obj(file_obj):
-    path_on_disk = os.path.abspath(file_obj.location_on_disk)
-    if os.path.exists(path_on_disk):
-        return flask.send_file(path_on_disk)
-    return flask.abort(404)
-
 class GBServer(flask.Flask):
     """
     GBServer is the GitBuilding server it is a child class of flask.Flask. It can be
@@ -53,46 +47,44 @@ class GBServer(flask.Flask):
         self._read_config()
 
         # Two render objects, one for static one for live
-        self.renderer = render.GBRenderer(self._config)
+        self.renderer = render.GBRenderer(self._config, static=False)
+        # The live renderer shows the static rendering as static=False
+        # is used show the header buttons.
         self.live_renderer = render.GBRenderer(deepcopy(self._config))
         self._unsaved_dropped_files = DroppedFiles()
         super(GBServer, self).__init__(__name__)
 
         # Define URL rules!
         self.add_url_rule("/", "render", self._render_page)
-        self.add_url_rule("/render_markdown",
+        self.add_url_rule("/-/render_markdown",
                           "live_render",
                           self._live_render,
                           methods=["POST"])
         self.add_url_rule("/<path:subpath>", "render", self._render_page)
         self.add_url_rule("/assets/<path:subpath>", "assets", self._return_assets)
-        self.add_url_rule("/editor/", "editor", self._edit_page)
-        self.add_url_rule("/editor/save", "save", self._save_edit, methods=["POST"])
-        self.add_url_rule("/editor/raw", "raw", self._raw_md)
-        self.add_url_rule("/editor/<path:subpath>",
-                          "editor_redirect",
-                          self._editor_redirect)
-        self.add_url_rule("/<path:subpath>/editor/", "editor", self._edit_page)
-        self.add_url_rule("/<path:subpath>/editor/raw", "raw", self._raw_md)
-        self.add_url_rule("/<path:subpath>/editor/save",
+        self.add_url_rule("/-/editor/", "editor", self._edit_page)
+        self.add_url_rule("/-/editor/save", "save", self._save_edit, methods=["POST"])
+        self.add_url_rule("/-/editor/raw", "raw", self._raw_md)
+        self.add_url_rule("/-/create-homepage/", "create_homepage", self._create_homepage)
+        self.add_url_rule("/-/contents-page", "contents_page", self._contents_page)
+
+        self.add_url_rule("/<path:subpath>/-/editor/", "editor", self._edit_page)
+        self.add_url_rule("/<path:subpath>/-/editor/raw", "raw", self._raw_md)
+        self.add_url_rule("/<path:subpath>/-/editor/save",
                           "save",
                           self._save_edit,
                           methods=["POST"])
 
-        self.add_url_rule("/<path:_>/editor/dropped-file",
+        self.add_url_rule("/<path:subpath>/-/editor/dropped-file",
                           "droppedfile",
                           self._dropped_file,
                           methods=["POST"])
 
-        self.add_url_rule("/editor/dropped-file",
+        self.add_url_rule("/-/editor/dropped-file",
                           "droppedfile",
                           self._dropped_file,
                           methods=["POST"])
 
-        self.add_url_rule("/<path:_>/editor/<path:subpath>",
-                          "editor_redirect",
-                          self._editor_redirect)
-
     def _read_config(self):
         """
         Reads the project data generated when converting BuildUp to markdown
@@ -118,7 +110,7 @@ class GBServer(flask.Flask):
         Returns this in JSON
         """
         md = ''
-        if subpath is None and request.path == "/editor/raw":
+        if subpath is None and request.path == "/-/editor/raw":
             if self.doc.config.landing_page is not None:
                 md = self.doc.landing_page.get_raw()
                 return jsonify({"md": md})
@@ -177,6 +169,15 @@ class GBServer(flask.Flask):
         except IOError:
             return False
 
+    def _create_homepage(self):
+        md = "# Project title\nSome text for this page"
+        self._save_page('index', md)
+        self._rebuild_docs()
+        return flask.redirect('/-/editor/')
+
+    def _contents_page(self):
+        return self.renderer.contents_page(self.doc.output_files)
+
     def _rebuild_docs(self):
         file_list = read_directory('.', exclude_list=self.doc.config.exclude)
         if self._license_file is not None:
@@ -186,12 +187,18 @@ class GBServer(flask.Flask):
         self.renderer.config = self._config
         self.renderer.populate_vars()
 
-    def _dropped_file(self, _=None):
+    def _dropped_file(self, subpath=None):
         """
         This gets run if a file gets dragged and dropped into the editor
         """
         files = request.files
-        # loop through all but return after first image.
+        if subpath is None:
+            folder_depth = 0
+        else:
+            folder_depth = len(subpath.split('/')) - 1
+        out_filenames = []
+        md_line = ''
+        # loop through all files and save images
         for file in files:
             if files[file].mimetype.startswith("image"):
                 files[file].filename = files[file].filename.replace(" ", "")
@@ -208,10 +215,16 @@ class GBServer(flask.Flask):
                 temp_path = os.path.join(gettempdir(), str(uuid())+ext)
                 files[file].save(temp_path)
                 self._unsaved_dropped_files.add_file(file_path, temp_path)
-                return jsonify({"received": True, "filename": file_path})
-            _LOGGER.warning("Cannot upload file of mimetype: %s", files[file].mimetype)
+                out_filenames.append(file_path)
+                md_file_path = '../'*folder_depth + file_path
+                md_line += f'![]({md_file_path})\n'
+            else:
+                _LOGGER.warning("Cannot upload file of mimetype: %s", files[file].mimetype)
 
-        # if not returned yet nothing was an image
+        if len(out_filenames) > 0:
+            return jsonify({"received": True,
+                            "filenames": out_filenames,
+                            "md_line": md_line})
         return flask.abort(405)
 
     def _live_render(self):
@@ -227,17 +240,16 @@ class GBServer(flask.Flask):
         if not "page" in content: # Live render landing page
             if self.doc.config.landing_page is not None:
                 page = self.doc.landing_page
-                processed_text = page.rebuild(content["md"])
+                processed_text = page.rebuild(content["md"], '-/editor/index')
                 title = page.title
                 self.live_renderer.config.title = title
                 self.live_renderer.populate_vars()
-            else:
-                processed_text = ""
         else:
             page = self._get_docpage(content["page"])
             if page is None:
                 return jsonify({"html": "", "log": "", "number": 0})
-            processed_text = page.rebuild(content["md"])
+            processed_text = page.rebuild(content["md"], content["page"]+'/-/editor/index')
+
         html = self.live_renderer.render_md(processed_text,
                                             page.filepath,
                                             fullpage=False,
@@ -253,7 +265,7 @@ class GBServer(flask.Flask):
         Starts the live editor for a particular page
         """
 
-        if (subpath is None and request.path == "/editor/") or os.path.splitext(subpath)[1] == '':
+        if (subpath is None and request.path == "/-/editor/") or os.path.splitext(subpath)[1] == '':
             self.live_renderer.config = deepcopy(self._config)
             self.live_renderer.populate_vars()
 
@@ -264,18 +276,10 @@ class GBServer(flask.Flask):
         html = self.renderer.render("<h1>Sorry. Cannot edit this file!</h1>")
         return html
 
-    def _editor_redirect(self, subpath=None, _=None):
-        """
-        Relative links inside the editor will go to here instead of their intended
-        destination. This redirects them to the correct place.
-        """
-        return flask.redirect(f"/{subpath}")
-
     def _render_page(self, subpath=None):
         """
         Renders the static version of a page
         """
-
         #remove any reference to .html as it confuses things
         if subpath is not None and os.path.splitext(subpath)[1] == '.html':
             subpath = os.path.splitext(subpath)[0]
@@ -288,7 +292,7 @@ class GBServer(flask.Flask):
             if self.doc.config.landing_page is not None:
                 translated_path = self.doc.config.landing_page
             else:
-                return self.renderer.render_md("No homepage set", editorbutton=False)
+                return self.renderer.empty_homepage()
         elif os.path.splitext(subpath)[1] == '':
             translated_path = os.path.splitext(subpath)[0] + '.md'
         elif os.path.splitext(subpath)[1] == '.md':
@@ -304,7 +308,7 @@ class GBServer(flask.Flask):
                 return self.renderer.render_md(outfile.content,
                                                subpath,
                                                editorbutton=editorbutton)
-            return _send_file_obj(outfile)
+            return self._send_file_obj(outfile)
 
         if translated_path.endswith(".md"):
             outfile = self.doc.get_file(translated_path[:-2] + 'stl')
@@ -312,7 +316,7 @@ class GBServer(flask.Flask):
                 return self.renderer.stl_page(outfile.path)
 
             return self.renderer.render_md("# Page not found\n Do you want to "
-                                           f"[create it](/{subpath}/editor)",
+                                           f"[create it](/{subpath}/-/editor)",
                                            subpath,
                                            editorbutton=True)
         # For missing files that are not mark down check the temporary
@@ -325,8 +329,21 @@ class GBServer(flask.Flask):
         file_list = read_directory('.', exclude_list=self._config.exclude)
         if subpath in file_list:
             file_obj = file_list[file_list.index(subpath)]
-            return _send_file_obj(file_obj)
+            return self._send_file_obj(file_obj)
+        return self._404_or_missing_image(subpath)
 
+    def _send_file_obj(self, file_obj):
+        path_on_disk = os.path.abspath(file_obj.location_on_disk)
+        if os.path.exists(path_on_disk):
+            return flask.send_file(path_on_disk)
+        return self._404_or_missing_image(file_obj.path)
+
+    def _404_or_missing_image(self, subpath):
+        if subpath.lower().endswith((".jpg", ".jpeg", ".png", ".gif", '.svg',
+                                     ".tif", ".tiff", '.webp')):
+            gbpath = os.path.dirname(__file__)
+            image = os.path.join(gbpath, "static", "local-server", "Missing_image.png")
+            return flask.send_file(image)
         return flask.abort(404)
 
     def _return_assets(self, subpath):
@@ -391,7 +408,7 @@ class DevServer(GBServer):
         """
         Starts the live editor for a particular page
         """
-        if (subpath is None and request.path == "/editor/") or os.path.splitext(subpath)[1] == '':
+        if (subpath is None and request.path == "/-/editor/") or os.path.splitext(subpath)[1] == '':
             self.live_renderer.config = deepcopy(self._config)
             self.live_renderer.populate_vars()
 
