@@ -17,8 +17,8 @@ class URLRulesTestCase(unittest.TestCase):
         self.assertTrue(isinstance(translator, URLTranslator))
 
     def test_modifier(self):
-        def mod(url):
-            return url[:-3]
+        def mod(url, anchor):
+            return url[:-3], anchor
         rules = copy(self.root_rules)
         rules.add_modifier(mod)
         translator = rules.create_translator('dir/index.md')
